package jartege ; 

import java.util.* ; 

/**
 * This class allows one to choose random values of primitive types.
 *
 * @author Catherine Oriat
 */

public class RandomValue {

   private RandomValue() { } 

   static private Random alea = new Random() ; 


   // ------------------------------------------------------------------------
   // Values of type boolean
   // ------------------------------------------------------------------------

   /**
    * Random value of type boolean.
    */

   public static boolean booleanValue() {
      return alea.nextBoolean() ; 
   }

   // ------------------------------------------------------------------------
   // Values of type byte
   // ------------------------------------------------------------------------

   /**
    * Random value of type byte.
    */

   public static byte byteValue() {
      return (byte) byteValue(Byte.MIN_VALUE, Byte.MAX_VALUE) ; 
   }

   /**
    * Random value of type byte in range [0, max].
    *
    * <pre><jml>
    * 
    * requires
    *    0 <= max ; 
    *
    * ensures
    *    0 <= \result && \result <= max ; 
    *
    * </jml></pre>
    */

   public static byte byteValue(byte max) {
      return (byte) byteValue((byte) 0, max) ;
   }

   /**
    * Random value of type byte in range [min, max].
    *
    * <pre><jml>
    * 
    * requires
    *    min <= max ; 
    *
    * ensures
    *    min <= \result && \result <= max ;
    *
    * </jml></pre>
    */

   public static byte byteValue(byte min, byte max) {
      return (byte) (alea.nextInt(1 + max - min) + min) ; 
   }

   /**
    * Random value of type byte chosen in the specified array of bytes.
    *
    * <pre><jml>
    * 
    * requires
    *    array != null && array.length != 0 ; 
    *
    * ensures
    *    (\exists int i ; 0 <= i && i < array.length ; \result == array[i]) ; 
    *
    * </jml></pre>
    */

   public static byte byteValue(byte[] array) {
      int val = alea.nextInt(array.length) ; 
      return array[val] ; 
   }



   // ------------------------------------------------------------------------
   // Values of type char
   // ------------------------------------------------------------------------

   /**
    * Random value of type char.
    */

   public static char charValue() {
      return (char) charValue(Character.MIN_VALUE, Character.MAX_VALUE) ;
   }

   /**
    * Random value of type char in range [0, max].
    *
    * <pre><jml>
    * 
    * requires
    *    0 <= max ; 
    *
    * ensures
    *    0 <= \result && \result <= max ; 
    *
    * </jml></pre>
    */
   
   public static char charValue(char max) {
      return (char) charValue((char) 0, max) ; 
   }

   /**
    * Random value of type char in range [min, max].
    *
    * <pre><jml>
    * 
    * requires
    *    min <= max ; 
    *
    * ensures
    *    min <= \result && \result <= max ;
    *
    * </jml></pre>
    */

   public static char charValue(char min, char max) {
      return (char) (alea.nextInt(1 + max - min) + min) ; 
   }

   /**
    * Random value of type char chosen in the specified array.
    *
    * <pre><jml>
    * 
    * requires
    *    array != null && array.length != 0 ; 
    *
    * ensures
    *    (\exists int i ; 0 <= i && i < array.length ; \result == array[i]) ; 
    *
    * </jml></pre>
    */

   public static char charValue(char[] array) {
      int val = alea.nextInt(array.length) ; 
      return array[val] ; 
   }

   /**
    * Random value of printable character (in range [32, 127]).
    *
    * <pre><jml>
    *
    * ensures
    *    32 <= \result && \result <= 127 ;
    *
    * </jml></pre>
    */
   
   public static char printableCharValue() {
      return printableCharValue((char) 32, (char) 127) ; 
   }

   /**
    * Random value of printable character in range [32, max].
    *
    * <pre><jml>
    *
    * requires
    *    32 <= max && max <= 127 ; 
    *
    * ensures
    *    32 <= \result && \result <= max ; 
    *
    * </jml></pre>
    */
   
   public static char printableCharValue(char max) {
      return printableCharValue((char) 32, max) ; 
   }


   /**
    * Random value of printable character in range [min, max].
    *
    * <pre><jml>
    *
    * requires
    *    32 <= min && min <= max && max <= 127 ;
    *
    * ensures
    *    min <= \result && \result <= max ;
    *
    * </jml></pre>
    */
   
   public static char printableCharValue(char min, char max) {
      return (char) (alea.nextInt(1 + max - min) + min) ; 
   }

   // ------------------------------------------------------------------------
   // Values of type double
   // ------------------------------------------------------------------------

   /**
    * Random value of type double in range [min, max], 
    * for min >= 0 and max >= 0.
    * Note that the result is not uniformely distributed in range
    * [min, max].
    *
    * <pre><jml>
    *
    * requires 
    *    min >= 0 && max >= 0 && min <= max ; 
    *
    * ensures 
    *    min <= \result && \result <= max ; 
    *
    * </jml></pre>
    */

   private static double nonUniformDoubleValuePos(double min, double max) {
      if (min > max) {
         return min ; 
      }
      long bitsMin = Double.doubleToLongBits(min) ; 
      long bitsMax = Double.doubleToLongBits(max) ; 
      long expMin = FloatingValue.exponent(bitsMin) ; 
      long expMax = FloatingValue.exponent(bitsMax) ; 
      long diff = expMax - expMin ; 
      long mantMin = FloatingValue.mantissa(bitsMin) ; 
      long mantMax = FloatingValue.mantissa(bitsMax) ; 
      long exp ; 
      long mant ; 
      double result ; 
      if (diff == 0) {
         exp = expMin ; 
         mant = longValue(mantMin, mantMax) ;
         result = FloatingValue.createDouble(0, mant, exp) ;
      } else if (diff == 1) {
         double val = doubleValue(1) ; 
         double d1 = (double) mantMax ; 
         double d2 = (double) ((1L << 52) - mantMin) ; 
         double seuil = d2 / (d1 + d2) ; 
         if (val >= seuil) {
            exp = expMin ; 
            mant = longValue(mantMin, (1L << 52) - 1) ; 
         } else {
            exp = expMax ; 
            mant = longValue(0, mantMax) ; 
         }
         result = FloatingValue.createDouble(0, mant, exp) ;
      } else {
         boolean found = false ; 
         result = 0 ; 
         while (!found) {
            exp = longValue(expMin, expMax) ;
            mant = longValue(0, (1L << 52L) - 1L) ; 
            result = FloatingValue.createDouble(0, mant, exp) ;
            found = min <= result & result <= max ; 
         }
      }
      return result ; 
   }

   /**
    * Random value of type double in range [min, max].
    * A sign, mantissa and exponent are randomly chosen and put 
    * together to form a double number.
    * Note that the result is not uniformely distributed in 
    * [min, max].
    *
    * <pre><jml>
    *
    * requires 
    *    min <= max ; 
    *
    * ensures 
    *    min <= \result && \result <= max ; 
    *
    * </jml></pre>
    */

   static public double nonUniformDoubleValue(double min, double max) {
      double result ; 
      if (max >= 0) {
         if (min >= 0) { 
            //@ assert max >= 0 && min >= 0 ; 
            result = nonUniformDoubleValuePos(min, max) ; 
         } else {
            //@ assert max >= 0 && min <= 0 ; 
            double f = alea.nextDouble() ; 
            if (f >= min / (min - max)) {
               result = nonUniformDoubleValuePos(0, max) ; 
            } else {
               result = -nonUniformDoubleValuePos(0, -min) ; 
            }
         }
      } else {
         //@ assert min <= 0 && max <= 0 ; 
         result = -nonUniformDoubleValuePos(-max, -min) ; 
      }
      return result ; 
   }
      
   /**
    * Random value of type double in range [0, max].
    * A mantissa and an exponent are randomly chosen and put 
    * together to form a double number less or equal to max.
    * Note that the result is not uniformely distributed in range
    * [0, max].
    *
    * <pre><jml>
    *
    * requires 
    *    max >= 0 ; 
    *
    * ensures 
    *    0 <= \result && \result <= max ; 
    *
    * </jml></pre>
    */

   static public double nonUniformDoubleValue(double max) {
      return nonUniformDoubleValue(0, max) ; 
   }

   /**
    * Random value of type double. Does not generate infinity nor NaN.
    * A sign, an exponent and a mantissa are randomly (and uniformely) chosen
    * and put together to form a double number.
    * Note that the result is not uniformely distributed in double.
    */

   static public double nonUniformDoubleValue() {
      long sign = longValue(0, 1) ;
      long mantissa = longValue(0, (1L << 52) - 1) ; 
      long exponent = longValue(0, 2046) ; 
      return FloatingValue.createDouble(sign, mantissa, exponent) ; 
   }


   /**
    * Random double value in range [min, max].
    *
    * <pre><jml>
    *
    * requires
    *    min <= max ; 
    *
    * ensures 
    *    min <= \result && \result <= max ; 
    *
    * </jml></pre>
    */

   static public double doubleValue(double min , double max) {
      double result ; 
      if (min <= 0) {
         if (max <= 0) {
            //@ assert min <= 0 && max <= 0 ; 
            double f = alea.nextDouble() ; 
            result = f * (max - min) + min ; 
         } else {
            //@ assert min <= 0 && max >= 0 ; 
            double f = alea.nextDouble() ;
            if (f > min / (min - max)) {
               result = doubleValue(max) ; 
            } else {
               result = -doubleValue(-min) ; 
            }
         }
      } else {
         //@ assert min >= 0 && max >= 0 ; 
         double f = alea.nextDouble() ;
         result = f * (max - min) + min ;
      }
      return result ; 
   }

   /**
    * Random double value in range [0, max].
    *
    * <pre><jml>
    *
    * requires 
    *    max >= 0 ; 
    *
    * ensures
    *    0 <= \result && \result <= max ; 
    *
    * </jml></pre>
    */

   static public double doubleValue(double max) {
      return alea.nextDouble() * max ; 
   }


   // ------------------------------------------------------------------------
   // Values of type int
   // ------------------------------------------------------------------------

   /**
    * Random value of type int.
    */

   static public int intValue() {
      return alea.nextInt() ; 
   }

   /**
    * Random value chosen in an array of int.
    *
    * <pre><jml>
    * 
    * requires
    *    array != null && array.length != 0 ; 
    *
    * ensures
    *    (\exists int i ; 0 <= i && i < array.length ; \result == array[i]) ; 
    *
    * </jml></pre>
    */

   static public int intValue(int[] array) {
      int taille = array.length ; 
      int val = alea.nextInt(taille) ; 
      return array[val] ; 
   }

   /**
    * Random value of type int in range [min, max].
    *
    * <pre><jml>
    *
    * requires
    *    min <= max ; 
    *
    * ensures
    *    min <= \result && \result <= max ;  
    *
    * </jml></pre>
    */
    
   static public int intValue(int min, int max) {
      double d = alea.nextDouble() ; 
      int result = (int) Math.floor(d * (1.0 + max - min) + min) ; 
      return result ; 
   }

   /**
    * Random value of type int in range [0, max].
    *
    * <pre><jml>
    *
    * requires
    *    0 <= max ;
    *
    * ensures
    *    0 <= \result && \result <= max ;
    *
    * </jml></pre>
    */

   static public int intValue(int max) {
      return intValue(0, max) ; 
   }


   // ------------------------------------------------------------------------
   // Values of type long
   // ------------------------------------------------------------------------
   
   /**
    * Random value of type long.
    */

   public static long longValue() {
      return alea.nextLong() ; 
   }

   /**
    * Random value of type long.
    *
    * <pre><jml>
    *
    * requires
    *    0 <= max ;
    *
    * ensures
    *    0 <= \result && \result <= max ;
    *
    * </jml></pre>
    */
   
   public static long longValue(long max) {
      return longValue(0, max) ; 
   }

   /**
    * Random value of type long.
    *
    * <pre><jml>
    *
    * requires
    *    min <= max ;
    *
    * ensures
    *    min <= \result && \result <= max ;
    *
    * </jml></pre>
    */

   public static long longValue(long min, long max) {
      double d = alea.nextDouble() ; 
      double diff = 1.0 + max - min ; 
      long result = (long) Math.floor(d * diff) + min ; 
      return result ; 
   }

   /**
    * Random value chosen in an array of long.
    *
    * <pre><jml>
    * 
    * requires
    *    array != null && array.length != 0 ; 
    *
    * ensures
    *    (\exists int i ; 0 <= i && i < array.length ; \result == array[i]) ; 
    *
    * </jml></pre>
    */

   static public long longValue(long[] array) {
      int taille = array.length ; 
      int val = alea.nextInt(taille) ; 
      return array[val] ; 
   }


   // ------------------------------------------------------------------------
   // Values of type short
   // ------------------------------------------------------------------------

   /**
    * Random value of type short.
    */

   public static short shortValue() {
      return (short) (alea.nextInt(1 + Short.MAX_VALUE - Short.MIN_VALUE) 
                      + Short.MIN_VALUE) ; 
   }

   /**
    * Random value of type short in range [min, max].
    *
    * <pre><jml>
    *
    * requires 
    *    min <= max ; 
    *
    * ensures 
    *    min <= \result && \result <= max ; 
    *
    * </jml></pre>
    */

   public static short shortValue(short min, short max) {
      return (short) (alea.nextInt(1 + max - min) + min) ; 
   }

   /**
    * Random value of type short in range [0, max].
    *
    * <pre><jml>
    *
    * requires
    *    0 <= max ;
    *
    * ensures
    *    0 <= \result && \result <= max ;
    *
    * </jml></pre>
    */

   public static short shortValue(short max) {
      return (short) alea.nextInt(1 + max) ;
   }


   /**
    * Random value chosen in an array of short.
    *
    * <pre><jml>
    * 
    * requires
    *    array != null && array.length != 0 ; 
    *
    * ensures
    *    (\exists int i ; 0 <= i && i < array.length ; \result == array[i]) ; 
    *
    * </jml></pre>
    */

   public static short shortValue(short[] array) {
      int val = alea.nextInt(array.length) ; 
      return array[val] ; 
   }


   // ------------------------------------------------------------------------
   // Values of type float
   // ------------------------------------------------------------------------

   /**
    * Random value of type float in range [min, max], 
    * for min >= 0 and max >= 0.
    *
    * <pre><jml>
    *
    * requires 
    *    min >= 0 && max >= 0 && min <= max ; 
    *
    * ensures 
    *    min <= \result && \result <= max ; 
    *
    * </jml></pre>
    */

   private static float nonUniformFloatValuePos(float min, float max) {
      if (min > max) {
         return min ; 
      }
      int bitsMin = Float.floatToIntBits(min) ; 
      int bitsMax = Float.floatToIntBits(max) ; 
      int expMin = FloatingValue.exponent(bitsMin) ; 
      int expMax = FloatingValue.exponent(bitsMax) ; 
      int diff = expMax - expMin ; 
      int mantMin = FloatingValue.mantissa(bitsMin) ; 
      int mantMax = FloatingValue.mantissa(bitsMax) ; 
      int exp ; 
      int mant ;
      float result ; 
      
      if (diff == 0) {
         exp = expMin ; 
         mant = intValue(mantMin, mantMax) ;
         result = FloatingValue.createFloat(0, mant, exp) ;
      } else if (diff == 1) {
         float val = floatValue(1) ; 
         float d1 = (float) mantMax ; 
         float d2 = (float) ((1 << 23) - mantMin) ; 
         float seuil = d2 / (d1 + d2) ; 
         if (val >= seuil) {
            exp = expMin ; 
            mant = intValue(mantMin, (1 << 23) - 1) ;
         } else {
            exp = expMax ; 
            mant = intValue(0, mantMax) ;
         }
         result = FloatingValue.createFloat(0, mant, exp) ;
      } else {
         boolean found = false ; 
         result = 0 ; 
         while (!found) {
            exp = intValue(expMin, expMax) ; 
            mant = intValue(0, (1 << 23) - 1) ; 
            result = FloatingValue.createFloat(0, mant, exp) ; 
            found = min <= result && result <= max ; 
         }
      }
      return result ; 
   }

   /**
    * Random value of type float in range [min, max].
    * A sign, mantissa and exponent are randomly chosen and put 
    * together to form a float number.
    * Note that the result is not uniformely distributed in 
    * [min, max].
    *
    * <pre><jml>
    *
    * requires 
    *    min <= max ; 
    *
    * ensures 
    *    min <= \result && \result <= max ; 
    *
    * </jml></pre>
    */

   static public float nonUniformFloatValue(float min, float max) {
      float result ; 
      if (max >= 0) {
         if (min >= 0) { 
            //@ assert max >= 0 && min >= 0 ; 
            result = nonUniformFloatValuePos(min, max) ; 
         } else {
            //@ assert max >= 0 && min <= 0 ; 
            float f = alea.nextFloat() ; 
            if (f >= min / (min - max)) {
               result = nonUniformFloatValuePos(0, max) ; 
            } else {
               result = -nonUniformFloatValuePos(0, -min) ; 
            }
         }
      } else {
         //@ assert min <= 0 && max <= 0 ; 
         result = -nonUniformFloatValuePos(-max, -min) ; 
      }
      return result ; 
   }
      
   /**
    * Random value of type float in range [0, max].
    * A mantissa and an exponent are randomly chosen and put 
    * together to form a float number less or equal to max.
    * Note that the result is not uniformely distributed in [0, max[.
    *
    * <pre><jml>
    *
    * requires 
    *    max >= 0 ; 
    *
    * ensures 
    *    0 <= \result && \result <= max ; 
    *
    * </jml></pre>
    */

   static public float nonUniformFloatValue(float max) {
      return nonUniformFloatValue(0, max) ; 
   }

   /**
    * Random value of type float. Does not generate infinity nor NaN.
    * A sign, an exponent and a mantissa are randomly (and uniformely) chosen
    * and put together to form a float number.
    * Note that the result is not uniformely distributed in float.
    */

   static public float nonUniformFloatValue() {
      int sign = intValue(0, 1) ; 
      int mantissa = intValue(0, (1 << 23) - 1) ; 
      int exponent = intValue(0, 254) ; 
      return FloatingValue.createFloat(sign, mantissa, exponent) ; 
   }

   /**
    * Random float value in range [min, max].
    *
    * <pre><jml>
    *
    * requires
    *    min <= max ; 
    *
    * ensures 
    *    min <= \result && \result <= max ; 
    *
    * </jml></pre>
    */

   static public float floatValue(float min , float max) {
      float result ; 
      if (min <= 0) {
         if (max <= 0) {
            //@ assert min <= 0 && max <= 0 ; 
            float f = alea.nextFloat() ; 
            result = f * (max - min) + min ; 
         } else {
            //@ assert min <= 0 && max >= 0 ; 
            float f = alea.nextFloat() ;
            if (f > min / (min - max)) {
               result = floatValue(max) ; 
            } else {
               result = -floatValue(-min) ; 
            }
         }
      } else {
         //@ assert min >= 0 && max >= 0 ; 
         float f = alea.nextFloat() ;
         result = f * (max - min) + min ;
      }
      return result ; 
   }

   /**
    * Random float value in range [0, max].
    *
    * <pre><jml>
    *
    * requires 
    *    max >= 0 ; 
    *
    * ensures
    *    0 <= \result && \result <= max ; 
    *
    * </jml></pre>
    */

   static public float floatValue(float max) {
      return alea.nextFloat() * max ; 
   }


   // ------------------------------------------------------------------------
   // Values of type String
   // ------------------------------------------------------------------------

   /**
    * Random string chosen in an array of String.
    *
    * <pre><jml>
    *
    * requires
    *    array != null && array.length != 0 ; 
    *
    * ensures
    *    (\exists int i ; 0 <= i && i < array.length ; 
    *                     \result == array[i]) ; 
    *
    * </jml></pre>
    */

   static public String stringValue(String[] array) {
      int taille = array.length ; 
      int val = alea.nextInt(taille) ; 
      return array[val] ; 
   }

   /**
    * Random String.
    */

   static public String stringValue() {
      int length = intValue(-1, 255) ; 
      if (length == -1) {
         return null ; 
      } else {
         char[] array = new char[length] ; 
         for (int i = 0 ; i < array.length ; i++) {
            char c = charValue() ;
            array[i] = c ; 
         }
         return new String(array) ; 
      }
   }

   /**
    * Random String of printable characters of length in range [min, max].
    *
    * <pre><jml>
    *
    * requires 
    *    0 <= min && min <= max ; 
    *
    * ensures
    *    min <= \result.length() && \result.length() <= max ; 
    *
    * </jml></pre>
    */

   static public String stringValue(short min, short max) {
      short length = shortValue(min, max) ; 
      char[] array = new char[length] ;
      for (short i = 0 ; i < array.length ; i++) {
         char c = charValue() ;
         array[i] = c ;
      }
      return new String(array) ;
   }
   

   /**
    * Random String of printable characters.
    */

   static public String printableStringValue() {
      int length = intValue(-1, 255) ; 
      if (length == -1) {
         return null ; 
      } else {
         char[] array = new char[length] ; 
         for (int i = 0 ; i < array.length ; i++) {
            char c = printableCharValue() ;
            array[i] = c ; 
         }
         return new String(array) ; 
      }
   }

   /**
    * Random String of printable characters of length in range [min, max].
    *
    * <pre><jml>
    *
    * requires 
    *    0 <= min && min <= max ; 
    *
    * ensures
    *    min <= \result.length() && \result.length() <= max ; 
    *
    * ensures
    *    (\forall int i ; 0 <= i && i < \result.length() ; 
    *       32 <= \result.charAt(i) && \result.charAt(i) <= 127) ; 
    *
    * </jml></pre>
    */

   static public String printableStringValue(short min, short max) {
      short length = shortValue(min, max) ; 
      char[] array = new char[length] ;
      for (short i = 0 ; i < array.length ; i++) {
         char c = printableCharValue() ;
         array[i] = c ;
      }
      return new String(array) ;
   }
   

   /**
    * Random object chosen in the specified collection.
    *
    * <pre><jml>
    *
    * requires
    *    coll != null && coll.size() != 0 ; 
    *
    * ensures
    *    coll.contains(\result) ; 
    *
    * </jml></pre>
    */

   static public Object oneObject(Collection coll) {
      int taille = coll.size() ; 
      Object result = null ; 
      if (taille != 0) {
         int valeur = intValue(taille - 1) ; 
         Iterator it = coll.iterator() ; 
         for (int i = 0 ; i < valeur ; i++) {
            it.next() ; 
         }
         result = it.next() ; 
      }
      return result ; 
   }

}

